﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinEditors;
using Infragistics.Win.UltraWinGrid;

namespace UltraGridColumnResizeWithDbNull
{
    public partial class Form1 : Form
    {
        private DataTable table;

        public Form1()
        {
            InitializeComponent();
            

        }

        private void Form1_Load(object sender, EventArgs e)
        {
            table = new DataTable();
            DataColumn col = new DataColumn("Cusip", typeof(String));
            table.Columns.Add(col);
            col = new DataColumn("MktPrice", typeof(double));
            col.Caption = "Market Price";
            table.Columns.Add(col);
            col = new DataColumn("Issuer", typeof(string));
            table.Columns.Add(col);

            for (int i = 1; i <= 50; i++)
            {
                DataRow row = table.NewRow();
                row["Cusip"] = "Cusip" + i.ToString();
                row["MktPrice"] = 100.0 + i / 10.0;
                row["Issuer"] = "Issuer " + i.ToString();
                table.Rows.Add(row);
            }
            ultraGrid1.DataSource = table;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            table.Rows[5][1] = DBNull.Value;
            setWidth();
        }

       
        private void button2_Click(object sender, EventArgs e)
        {
            table.Rows[5][1] = 100.0 + 5/10.0;
            setWidth();
        }

        private void setWidth()
        {
            int width = ultraGrid1.DisplayLayout.Bands[0].Columns[1].CalculateAutoResizeWidth(Infragistics.Win.UltraWinGrid.PerformAutoSizeType.AllRowsInBand, false);
            labelColumnWidth.Text = width.ToString();
            ultraGrid1.DisplayLayout.Bands[0].Columns[1].Width = width;
        }

        private void ultraGrid1_InitializeLayout(object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e)
        {
            CurrencyFieldFilter filter = new CurrencyFieldFilter(1.0f, "#,0.00", 5);
            UltraNumericEditor editor = getNumericEditor("lf", "#,0.00", filter);
            UltraGridColumn column = ultraGrid1.DisplayLayout.Bands[0].Columns[1];    
            column.Editor = null;                
            column.EditorControl = editor;                
            column.UseEditorMaskSettings = true;                
        }

        private UltraNumericEditor getNumericEditor(string dataType, string format, IEditorDataFilter filter)
        {
            UltraNumericEditor editor = new UltraNumericEditor();
            if ((dataType == "ld" || dataType == "d") && !format.Contains(".0"))
                editor.NumericType = NumericType.Integer;
            else
                editor.NumericType = NumericType.Double;

            editor.MaskInput = "{double:-8.3}";
            editor.MaskDisplayMode = Infragistics.Win.UltraWinMaskedEdit.MaskMode.IncludeLiterals;
            editor.MaskClipMode = Infragistics.Win.UltraWinMaskedEdit.MaskMode.Raw;

            editor.DataFilter = filter;
            editor.PromptChar = ' ';

            return editor;
        }
    }

    public class CurrencyFieldFilter : IEditorDataFilter
    {
        public float ExchangeRate;
        private String columnFormat;
        public int ReportFieldId; 

        
        public CurrencyFieldFilter(float exchangeRate, String columnFormat, int fieldId)
        {
            ExchangeRate = exchangeRate;
            this.columnFormat = columnFormat;
            ReportFieldId = fieldId;
        }

        public Object Convert(EditorDataFilterConvertArgs args)
        {
            double currency = 0.0f;
            switch (args.Direction)
            {
                case ConversionDirection.EditorToDisplay:
                    args.Handled = true;

                    if (args.Value == DBNull.Value || args.Value == null)
                        return "";

                    return args.Value;

                case ConversionDirection.DisplayToEditor:
                    args.Handled = true;
                    return args.Value;

                case ConversionDirection.OwnerToEditor:
                    args.Handled = true;

                    if (args.Value == DBNull.Value || args.Value == null)
                        return null;//???

                    
                    if (args.Value.ToString() == "-9999")
                        return "N/A";

                                        
                    currency = System.Convert.ToDouble(args.Value) * ExchangeRate;

                    //Return numeric value instead of string since we switch the editor from text editor 
                    //to numeric editor in order for auto filter to work.
                    //ZF 9/17/08 Bug 16910.
                    return currency;//.ToString(columnFormat);

                case ConversionDirection.EditorToOwner:
                    args.Handled = true;

                    //ZF 4/25/08 handle N/A

                    //Since we switch the editor from TextEditor to NumericEditor, we have to convert the original value args.Value differently.
                    //ZF 9/17/08 Bug 16910.
                    string numS;
                    double numD;
                    if (args.Value is string)
                    {
                        numS = (string)args.Value;
                        numD = System.Convert.ToDouble(numS);
                    }
                    else
                    {
                        numD = System.Convert.ToDouble(args.Value);
                        numS = args.Value.ToString();
                    }

                    if (numS == "N/A" || numS == "n/a")
                        return -9999.0;

                    currency = numD / ExchangeRate;
                    return currency;

                default:
                    return null;
            }
        }
    }
}
